/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#define PILOT_PRECOMPILED_HEADERS_OFF
#include "Serial Port Tools.h"
#include <SysEvtMgr.h>


// Globals.

UInt16		gPortReference		= 0;
Boolean		gCommEstablished	= false;


// ---------------------------------------------------------------------------
//		 Comm_Initialize
// ---------------------------------------------------------------------------
// Opens the communications port.

Int16 Comm_Initialize (Int16 iPort, DWord iBaud)
{
	Int16				aError;
	SerSettingsType		aSettings;

	// Exit if communications have already been established.

	if (gCommEstablished)
		return (kError_CommOpen);

	// Clear the globals.

	gPortReference		= 0;
	gCommEstablished	= false;

	// Find the serial library.

	Comm_ClearError ();

	aError = SysLibFind ("Serial Library", &gPortReference);
	if (aError != 0)
		return (kError_CantOpenPort);

	// Open the serial port.

	if (iPort != kPort_Pilot)
		return (kError_InvalidPort);

	Comm_ClearError ();

	aError = SerOpen (gPortReference, iPort, iBaud);

	if (aError == serErrAlreadyOpen)
	{
		SerClose (gPortReference);
		return (kError_PortOpen);
	}
	else if (aError != 0)
		return (kError_CantOpenPort);

	// Set the serial attributes.

	Comm_ClearError ();

	aSettings.baudRate		= iBaud;
	aSettings.flags			= serSettingsFlagBitsPerChar8 | serSettingsFlagStopBits1 | serSettingsFlagRTSAutoM | serSettingsFlagCTSAutoM;
	aSettings.ctsTimeout	= serDefaultCTSTimeout;

	aError = SerSetSettings (gPortReference, &aSettings);

	if (aError != 0)
	{
		SerClose (gPortReference);
		return (kError_CantOpenPort);
	}

	gCommEstablished = true;

	return (0);
}


// ---------------------------------------------------------------------------
//		 Comm_Dispose
// ---------------------------------------------------------------------------
// Closes the communications port.

void Comm_Dispose (void)
{
	// Exit if communications have not been established.

	if (!gCommEstablished)
		return;

	// Close the serial port.

	Comm_ClearError ();

	SerClose (gPortReference);

	gPortReference = 0;

	// Clear the globals.

	gCommEstablished = false;
}


// ---------------------------------------------------------------------------
//		 Comm_ClearError
// ---------------------------------------------------------------------------
// Clears the current serial errors.

void Comm_ClearError (void)
{
	// Exit if communications have not been established.

	if (!gCommEstablished)
		return;

	SerClearErr (gPortReference);
}
