/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef _BANK_ROM_H_
#define _BANK_ROM_H_


class ROMBank
{
	public:
		static void				Initialize		(void);
		static void				Reset			(void);

		static uae_u32			GetLong			(uaecptr iAddress);
		static uae_u32			GetWord			(uaecptr iAddress);
		static uae_u32			GetByte			(uaecptr iAddress);
		static void				SetLong			(uaecptr iAddress, uae_u32 iLongValue);
		static void				SetWord			(uaecptr iAddress, uae_u32 iWordValue);
		static void				SetByte			(uaecptr iAddress, uae_u32 iByteValue);
		static int				ValidAddress	(uaecptr iAddress, uae_u32 iSize);
		static uae_u8*			GetRealAddress	(uaecptr iAddress);

		static uaecptr			GetMemoryStart	(void) { return kMemoryStart; }
		static int				IsPCInRAM		(void) { return m68k_getpc () < ROMBank::kMemoryStart; }
		static int				IsPCInROM		(void) { return m68k_getpc () >= ROMBank::kMemoryStart; }

	private:
		static void				InvalidAccess	(uaecptr iAddress, long size, Bool forRead);

		static AddressBank		fgAddressBank;
		static const uae_u32	kMemoryStart;
};


class FlashBank
{
	public:
		static void				Initialize		(void);
		static void				Reset			(void);

		static uae_u32			GetLong			(uaecptr iAddress);
		static uae_u32			GetWord			(uaecptr iAddress);
		static uae_u32			GetByte			(uaecptr iAddress);
		static void				SetLong			(uaecptr iAddress, uae_u32 iLongValue);
		static void				SetWord			(uaecptr iAddress, uae_u32 iWordValue);
		static void				SetByte			(uaecptr iAddress, uae_u32 iByteValue);
		static int				ValidAddress	(uaecptr iAddress, uae_u32 iSize);
		static uae_u8*			GetRealAddress	(uaecptr iAddress);

		static uaecptr			GetMemoryStart	(void) { return kMemoryStart; }

	private:
		static AddressBank		fgAddressBank;
		static const uae_u32	kMemoryStart;

		static int				fgState;
		static uaecptr			fgAddressOfInterest;
		static Bool				fgEraseIsSetup;
};


#endif /* _BANK_ROM_H_ */

