/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef _MISCELLANEOUS_H_
#define _MISCELLANEOUS_H_

class StMemory
{
	public:
		StMemory		(	char*	inPtr = NULL);

		StMemory		(	long	inSize,
							Bool	inClearBytes = false);

		~StMemory		();

		operator char*	()			{ return mPtr; }

		char*	Get		() const	{ return mPtr; }
		Bool	IsOwner	() const	{ return mIsOwner; }
		Bool	IsValid	() const	{ return (mPtr != NULL); }

		void	Adopt	(	char*	inPtr);
		char*	Release	() const;
		void	Dispose	();

	protected:
		char*			mPtr;
		mutable Bool	mIsOwner;

	private:
		StMemory	( const StMemory &inPointerBlock);
		StMemory&	operator = (const StMemory &inPointerBlock);
};


class StMemoryMapper
{
	public:
		StMemoryMapper	(const void* memory, long size);
		~StMemoryMapper	(void);

	private:
		const void*	fMemory;
};

struct AppInfo
{
	ULong		creator;
	ULong		type;
	UInt		version;
	LocalID		dbID;
	UInt 		cardNo;
	ULong		modDate;
	Word		dbAttrs;

	char		name[dmDBNameLength];

	Bool operator < (const AppInfo& other) const
		{ if (creator < other.creator) return true;
		  if (creator > other.creator) return false;
		  if (type < other.type) return true;
		  return false; }

	Bool operator == (const AppInfo& other) const
		{ return (creator == other.creator) &&
				 (type == other.type); }
};

typedef vector<AppInfo>	AppInfoList;

#ifdef __MSL_FIX_ITERATORS__
__MSL_FIX_ITERATORS__(AppInfo)
#endif


Bool	StartsWith				(const char* s, const char* pattern);
Bool	EndsWith				(const char* s, const char* pattern);

void	GetApplicationNames		(AppInfoList& appList);

Bool	IsExecutable			(DWord dbType, DWord dbCreator, Word dbAttrs);
Bool	IsVisible				(DWord dbType, DWord dbCreator, Word dbAttrs);

void	ResetCalibrationInfo	(void);
void	SetHotSyncUserName		(const char*);

uae_u32	NextPowerOf2			(uae_u32 x);
uae_u32	DateToDays				(uae_u32 year, uae_u32 month, uae_u32 day);


uaecptr GetFunctionAddress		(uae_u16 trapWord, uae_u32 extra = 0, Bool digDeep = false);
uaecptr GetLibFunctionAddress	(uae_u16 trapWord, Word refNum, Bool digDeep);
uaecptr GetSysFunctionAddress	(uae_u16 trapWord, uae_u32 extra, Bool digDeep);
uaecptr GetStdDispatchAddress	(uaecptr entryPt, uae_u32 regD2);
uaecptr GetFlpEmDispatchAddress	(uaecptr entryPt, uae_u32 regD2);

extern uae_u32	gLibErrorBase;

char*	GetTrapName				(uae_u16 trapWord, uae_u32 extra = 0, Bool digDeep = false);
void	FindTrapName			(uae_u16 trapWord, char* nameP, uae_u32 extra = sysInvalidRefNum, Bool digDeep = false);

void	FindFunctionName		(uaecptr addr,
								 char* nameP,
								 uaecptr* startAddrP = NULL,
								 uaecptr* endAddrP = NULL);
uaecptr	FindFunctionStart		(uaecptr addr);
uaecptr	FindFunctionEnd			(uaecptr addr);
void	CopyMacsbugName			(uaecptr eof, char* name);
Bool	EndOfFunctionSequence	(uaecptr addr);
uae_u8	MacsbugNameLength		(uaecptr& addr, Bool* isFixed);
Bool	ValidMacsbugChar		(uae_u8 ch);

void	GetHostTime				(long* hour, long* min, long* sec);
void	GetHostDate				(long* year, long* month, long* day);

#endif	// _MISCELLANEOUS_H_
