/***********************************************************************
 *
 * Copyright  Palm Computing 1997 -- All Rights Reserved
 *
 * PROJECT:			    Web Library header file
 *
 * FILE:				WebLib.h
 *
 * DESCRIPTION:	
 *	  
 * OBJECTIVE:		
 *
 * REVISION HISTORY:
 * 	 970908		Des		Initial version
 *	 980119		tgh		New event structure
 *	 980120		tgh		Added TitleChange event
 *	 980429		des		Added dataP to webLaunchURL event(for POST)
 *
 **********************************************************************/
 
#ifndef	__WEBLIB_H__
#define	__WEBLIB_H__

#include <SystemMgr.h>
#include <INetMgr.h>

#pragma mark Constants

// Turn this on to get rid of webP->sockH socket handling from WebLib.c.
// This implies an API change, because the application needs to close sockets.
#if 0
#define NO_WEBP_SOCKH
#endif

// Creator. Used for both the database that contains the Web Library and
//  it's features for the feature manager. 
#define	webCreator				'webl'			// The Net Library creator

// Web Library features have this creator 
#define	webLibFtrCreator		webCreator		// creatorID of Web Lib features. 

// Feature webCreator, #0 is the version number
// 0xMMmfsbbb, where MM is major version, m is minor version
// f is bug fix, s is stage: 3-release,2-beta,1-alpha,0-development,
// bbb is build number for non-releases 
// V1.12b3   would be: 0x01122003
// V2.00a2   would be: 0x02001002
// V1.01     would be: 0x01013000
#define  webFtrNumVersion		0


// Database Type if the Web library. 
#define	webLibType				'libr'			// The Web Library Database type

// Name of the WebLib
#define	webLibName				"Web.lib"		// pass in to SysLibFind()

// WebLib Font values
#define webMaxFontSizes		5					// Maximum number of size increments <Sizes 0-4>
#define webMaxFontStyles	10					// Maximum number of style increments <Styles 0-9>


#pragma mark ErrorCodes

/********************************************************************
 * Error codes
 ********************************************************************/

#define	webErrBufTooSmall		(webErrorClass | 1)		// Buffer too small.
#define	webErrTypeInvalid		(webErrorClass | 2)		// Invalid type.
#define	webErrSourceInvalid		(webErrorClass | 3)		// Socket invalid or error on socket setup.
#define	webErrDestInvalid		(webErrorClass | 4)		// Window invalid or error on window setup.
#define	webErrCachingFailed		(webErrorClass | 5)		// Caching failed.
#define	webErrTooComplex		(webErrorClass | 6)		// Page too complex.
#define	webErrCMLEnd			(webErrorClass | 7)		// Page done.
#define	webErrCMLInvalid		(webErrorClass | 8)		// Invalid CML.
#define webErrUserCancel		(webErrorClass | 9)		// User initiated cancel
#define	webErrOutOfMemory		(webErrorClass | 10)	//
#define webErrBadHandle			(webErrorClass | 11)	// Invalid Handle
#define webErrParseOverrun		(webErrorClass | 12)	// Parsing past end of partial CML
#define webErrInvalidParam		(webErrorClass | 13)	// Invalid parameter
#define webErrCMLRestart		(webErrorClass | 14)	// Internal error
#define webErrCMLInvalidTable	(webErrorClass | 15) // Premature table tag
#define webErrCMLInvalidTD		(webErrorClass | 16) // Premature table cell begin tag
#define webErrCMLInvalidTDEnd	(webErrorClass | 17)	// Premature table cell end tag
#define webErrCMLInvalidSelect	(webErrorClass | 18) // Invalid select object type
#define webErrCMLInvalidEnd	(webErrorClass | 19)	// Invalid end tag
#define webErrCMLInvalidVersion	(webErrorClass | 20)	// Incompatable encoder version
#define webErrCMLInvalidNested	(webErrorClass | 21)	// Nested table (not allowed)

// MIME types for Web content
#define webContentTypeUnknown 	"Unknown"		// The content type is not known.
#define webContentTypeText 		"Text"			// The content type is ASCII. 
#define webContentTypeHTML 		"HTML"			// The content type is HTML.
#define webContentTypeCML 		"CML"			// The content type is CML. 
#define webContentTypePixmap 	"Pixmap"		// The content type is a compressed 2-bit pixmap.

// Render flags for source and destination
#define	webRenderSourceSock 	0x00000001	// The source handle represents a socket. 
#define webRenderSourceMem		0x00000002	// The source handle represents a memory region. 
#define webRenderDestWin		0x00000040	// The destination handle represents a window. 

// Cache flags for incoming web content
#define webCacheFlagsNoCache 		0x00000001	// Do not cache the content. 
#define webCacheFlagsOnce	 		0x00000002	// Store the item in the cache if it is not already in the cache. 
#define webCacheFlagsReplace 		0x00000004	// Store the item in the cache, overwriting any other entries with an equivalent URL. 
#define webCacheFlagsUseCached 		0x00000008	// If a cached value is available, use it. 
#define webCacheFlagsIgnoreCache 	0x00000010	// Invalidate any cached entries with this URL; fetch content from the network. 

/********************************************************************
 * Possible socket status values that can be returned
 ********************************************************************/
typedef enum {
	webStatusProgress,					// Percent complete
	webStatusTitleChange,				// Title change detected
	webStatusDone						// Done rendering document
	
	} WebStatusEnum;

// Web Rendering events

/********************************************************************
 * Structure of our Internet events. This structure is a superset of
 *  the regular event type. Note that we use the first 2 user events
 *  for the Internet Library so any app that uses this library must be
 *  to use user event IDs greater than WebRenderLastEvent.
 ********************************************************************/
enum webEvents {
	webRenderStatusChangeEvent = firstWebLibEvent,// Occurs when a rendering chunk completes
	webRenderLaunchURLEvent,					// Occurs when URL is launched from content area
	webRenderLastEvent							// First available user event
};

typedef struct {
	enum events		eType;
	Boolean			penDown;
	Byte				reserved;
	SWord			screenX;
	SWord			screenY;
	union {
		struct  {
			Word data1;
			Word data2;
			Word data3;
			Word data4;
			Word data5;
			Word data6;
			Word data7;
			Word data8;
			} generic;
			
		struct {
			Handle		renderID;				// render task identifier
			DWord		context;				// application defined
			Word		status;					// new status
			DWord		value;					// status code specific value
			} webStatusChange;
			
		// 5/1/98 - Removed context field so data would not be larger
		//  than 8 words
		struct {
			Handle		renderID;				// render task identifier
			BytePtr		urlP;					// requested URL
			BytePtr		dataP;					// extra data(e.g. for POST)
			Word		isIndex : 1;			// open URL as index
			Word		reserved : 15;			// must be zero
			} webLaunchURL;
		} data;
} WebEventType;

typedef WebEventType * WebEventPtr;





/***********************************************************************
 * Global defines for this module
 **********************************************************************/
typedef enum
{
	webSettingRenderImagesWhenWireline,
	webSettingRenderImagesWhenWireless,
	webSettingTitle,
	webSettingResponseURL,
	webSettingPageData,
	webSettingResponseSize,
	webSettingResponseCacheID
} WebSettingEnum;




/********************************************************************
 * Web Library traps. 
 ********************************************************************/
#pragma mark Traps
typedef enum {
	webLibTrapSettingGet = sysLibTrapCustom,
	webLibTrapSettingSet,

	webLibTrapFormHandleEvent,
	webLibTrapHandleEvent,

	webLibTrapRender,
	webLibTrapRenderCache,
	webLibTrapRenderCancel,

	webLibTrapCopyToClipboard,
	webLibTrapGetScrollData,
	webLibTrapScroll,
	
	webLibTrapCMLToSimpleText,
	webLibTrapSimpleTextToCML,
	
	webLibTrapWebLibSelectAll,
	webLibTrapWebLibRedrawForm,
	
	webLibTrapRenderComplete,
	webLibTrapRenderData,
	
	webLibTrapLast
	} WebLibTrapNumberEnum;


/***********************************************************************
 * Function declarations
 **********************************************************************/
 
Err	WebLibOpen(Word libRefnum, DWord flags, Word formID, Word baseFormID, 
				Word webGadgetID, Word webScrollBarID,
				FontID *webLibFontsP, Handle *websetHP)
		SYS_TRAP(sysLibTrapOpen);
Err	WebLibClose(Word libRefnum, Handle websetH)
		SYS_TRAP(sysLibTrapClose);

Err	WebLibSleep(Word libRefnum)
		SYS_TRAP(sysLibTrapSleep);
Err	WebLibWake(Word libRefnum)
		SYS_TRAP(sysLibTrapWake);


Err	WebLibSettingGet(Word libRefnum, Handle websetH, WebSettingEnum setting,
					VoidPtr bufP, WordPtr bufLenP)
				SYS_TRAP(webLibTrapSettingGet);
					
Err WebLibSettingSet(Word libRefnum, Handle websetH, WebSettingEnum setting,
					VoidPtr bufP, Word bufLen)
				SYS_TRAP(webLibTrapSettingSet);

Err WebLibFormHandleEvent (Word libRefnum, Word inetLibRefnum, Handle websetH, 
							Handle inetH, EventPtr eventP, Boolean *Handled)
				SYS_TRAP(webLibTrapFormHandleEvent);
				
Err WebLibHandleEvent(Word libRefnum, Word inetLibRefnum, Handle websetH,
					Handle inetH, EventPtr eventP)
				SYS_TRAP(webLibTrapHandleEvent);

Err WebLibRender(Word libRefnum, Word inetLibRefnum, Handle websetH,
				Handle inetsetH, BytePtr contentTypeP, BytePtr urlP,
				Handle sourceH, Handle destH, Byte renderFlags,
				Byte cacheFlags, ULongPtr renderID)
			SYS_TRAP(webLibTrapRender);
			
Err WebLibRenderCache(Word libRefnum, Handle webH, BytePtr urlP, ULong uniqueID)
				SYS_TRAP(webLibTrapRenderCache);

Err WebLibRenderCancel(Word libRefnum, Word inetLibRefnum, Handle websetH,
					Handle inetsetH, Handle renderID)
				SYS_TRAP(webLibTrapRenderCancel);

Err	WebLibRenderComplete(Word libRefnum, Handle websetH)
				SYS_TRAP(webLibTrapRenderComplete);

Err	WebLibRenderData(Word libRefnum, Handle webH, Handle sourceH)
				SYS_TRAP(webLibTrapRenderData);

Err WebLibCopyToClipboard (Word libRefnum, Handle websetH)
				SYS_TRAP(webLibTrapCopyToClipboard);

Err WebLibGetScrollData (Word libRefnum, Handle websetH,
						 Short *value, Short *min, Short *max, Short *pageSize)
				SYS_TRAP(webLibTrapGetScrollData);

Err WebLibScroll (Word libRefnum, Handle websetH, SWord moveBy, Boolean force)
				SYS_TRAP(webLibTrapScroll);

Err	WebLibCMLToSimpleText(Word libRefnum, Handle *dstHP, BytePtr srcP, DWordPtr srcBitOffsetP, DmOpenRef dbP)
				SYS_TRAP(webLibTrapCMLToSimpleText);

Err	WebLibSimpleTextToCML(Word libRefnum, Handle *dstHP, DWordPtr dstBitOffsetP, BytePtr srcP)
				SYS_TRAP(webLibTrapSimpleTextToCML);

Err WebLibSelectAll (Word libRefnum, Handle websetH)
				SYS_TRAP(webLibTrapWebLibSelectAll);

Err WebLibRedrawForm (Word libRefnum, Handle websetH)
				SYS_TRAP(webLibTrapWebLibRedrawForm);

#if EMULATION_LEVEL != EMULATION_NONE
/* This function is for debugging in the ClipperSim build */
#ifdef __cplusplus
extern "C"
#endif
Err	WebLibDumpCanvas(Word libRefnum, Handle websetH);
#endif

#endif 	//__WEBLIB_H__
