/* ================================================================================== */
/* Copyright (c) 1998-1999 3Com Corporation or its subsidiaries. All rights reserved. */
/* ================================================================================== */

#ifndef _TRAPPATCHES_H_
#define _TRAPPATCHES_H_


// Types.

enum CallROMType
{
	kExecuteROM,
	kSkipROM
};


// Function types for head- and Tailpatch functions.

typedef	CallROMType	(*HeadpatchProc)(void);
typedef	void		(*TailpatchProc)(void);


// Structure used to hold information about the currently
// running Palm OS application (as recorded when SysAppStartup
// is called).

struct EmuCurAppInfo
{
	DmOpenRef	fDB;
	UInt		fCardNo;
	LocalID		fDBID;
	UInt		fMemOwnerID;
	uaecptr		fStackP;
	uaecptr		fStackEndP;
	long		fStackSize;
	char		fName[dmDBNameLength];
	char		fVersion[256];	// <gulp> I hope this is big enough...
};


enum QuitStage
{
	kNoQuit,
	kWaitingForSysUIAppSwitch,
	kWaitingForSysAppStartup,
	kWaitingForSysAppExit,
	kTimeToQuit
};


class ModulePatchTable;


class Patches
{
	public:
		static void				Initialize				(void);
		static void				Reset					(void);
		static CallROMType		HandleSystemCall		(uae_u16 trapWord, uaecptr oldpc);
		static void				HandleTrap13			(uaecptr oldpc);
		static void				GetPatches				(HeadpatchProc& hp, TailpatchProc& tp, uae_u16 trapWord);
		static CallROMType		HandlePatches			(HeadpatchProc hp, TailpatchProc tp, uaecptr startPC, uae_u16);

		static void				InstallLibPatch			(const string& libName, UInt refNum, ModulePatchTable*& entry);

		static void				RemoveAllTailpatches	(void);
		static void				InstallAllTailpatches	(void);

		static CallROMType		CallHeadpatch			(HeadpatchProc);
		static void				CallTailpatch			(TailpatchProc);

		static DWord			OSVersion				(void);
		static DWord			OSMajorMinorVersion		(void);
		static DWord			OSMajorVersion			(void);
		static DWord			OSMinorVersion			(void);

		static Bool				AppInfoHasStackInfo		(void) { return OSMajorVersion () <= 2; }	// Changed in 3.0

		static void				SetSwitchApp			(UInt cardNo, LocalID dbID);
		static Err				SwitchToApp				(UInt cardNo, LocalID dbID);
		static void				QuitOnAppExit			(Bool);

		static void				PuppetString			(CallROMType& callROM,
														 Bool& clearTimeout);

			// These functions are used to inhibit Poser from display error
			// dialogs concerning certain improper practices.  If a system is
			// known to have a certain bug (as reported by the following
			// functions) and it looks like that bug is occuring, then we
			// inhibit the dialog.  Once the bug is fixed, however, we need
			// turn the error reporting back on.

			// Bugs fixed in 3.0.
		static Bool				HasFindShowResultsBug	(void) { return OSMajorVersion () <= 2; }
		static Bool				HasMenuHandleEventBug	(void) { return OSMajorVersion () <= 2; }
		static Bool				HasSysBinarySearchBug	(void) { return OSMajorVersion () <= 2; }

			// Bugs fixed in 3.1
		static Bool				HasGrfProcessStrokeBug	(void) { return OSMajorMinorVersion () <= 30; }

			// Bugs fixed in 3.2
		static Bool				HasBackspaceCharBug		(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasDeletedStackBug		(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasFindSaveFindStrBug	(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasFldDeleteBug			(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasFntDefineFontBug		(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasNetPrvTaskMainBug	(void) { return OSMajorMinorVersion () <= 31; }
		static Bool				HasNetPrvSettingSetBug	(void) { return OSMajorMinorVersion () <= 31; }

		static Bool				HasWellBehavedMemSemaphoreUsage (void);
			// ==== Done with bugs section ====

		static int				IsPCInMemMgr			(void) { return fgMemMgrCount > 0; }
		static void				EnterMemMgr				(const char*);
		static void				ExitMemMgr				(const char*);

		static int				IsInSysBinarySearch		(void) { return fgSysBinarySearchCount > 0; }

		static void				CollectCurrentAppInfo	(void);

	private:
		static void				SetupForTailpatch		(TailpatchProc, uaecptr oldpc);
		static TailpatchProc	RecoverFromTailpatch	(uaecptr oldpc);

	private:
		static long				fgMemMgrCount;

	private:
		static UInt				fgNextAppCardNo;
		static LocalID			fgNextAppDbID;
	public:	// SysHeadpatch::SysAppExit
		static Bool				fgQuitOnAppExit;
		static QuitStage		fgQuitStage;

	public:
		static long				fgMemSemaphoreCount;
		static unsigned long	fgMemSemaphoreReserveTime;
		static ULong			fgResizeOrigSize;
		static UInt				fgHeapID;				// MemPtrHeapID, MemHandleHeapID

	public:	// Accessed by SysHeadpatch::SysBinarySearch, SysTailpatch::SysBinarySearch
		static long				fgSysBinarySearchCount;

	public:	// Accessed by Errors::GetAppName && Errors::GetAppVersion

		static EmuCurAppInfo	fgCurAppInfo;

	public:	// Accessed by Hardware::PenEvent, Hardware::KeyboardEvent, and
			// UI code that sets up menus.

		static Bool				fgUIInitialized;
};


// ======================================================================
// At compile time, the list of functions we want to head- and tailpatch
// are stored in an array of ProtoPatchTableEntries.  At runtime, these
// compact arrays are expanded by ModuelPatchTable into a sparse array
// so that the function dispatch number can be used to look up the
// patch function.
// ======================================================================

struct ProtoPatchTableEntry
{
	uae_u16			fTrapWord;
	HeadpatchProc	fHeadpatch;
	TailpatchProc	fTailpatch;
};


// Defined in TrapPatches_NetLib.cpp, used in TrapPatches.cpp
extern ProtoPatchTableEntry	gProtoNetLibPatchTable[];

// Defined in TrapPatches_MemMgr.cpp, used in TrapPatches.cpp
extern ProtoPatchTableEntry	gProtoMemMgrPatchTable[];

#endif /* _TRAPPATCHES_H_ */

